## Alternative implementation of the estimator 
## using the same ID for original copy and replicate copy, i.e., some individuals have 4 rows, some have 8 rows

## need to pre-process the data set in a different way
## and adopt a user-defined correlation structure (varies with the desirable correlation structure, e.g., exch, ar1, etc.)
library(geepack)

GeeRepeatedMeasures2 <- function(dat, est.weight = FALSE, type, model=1, adjust.cov = 1, est.se = TRUE){
  ## GEE implementation of the estimator for repeated measures marginal model
  ## dat is generated from DataGenerator() and baseline covariates are pre-processed (standardized)
  ## est.weight is indicator of whether the known weights are estimated
  ## type=1 = indep working correlation
  ## type=2 = exch working correlation
  ## model indicates the marginal model that is used
  # model=1 = correct marginal model
  # model=2 = slope model (model each DTR to have linear trajectory)
  # model=3 = quadratic model (model each DTR to have quadratic trajectory)
  
  
  
  # adjust.cov indicates the type of adjustment used in estimating asymptotic SE
  # currently only have adjust.cov=1 = DF adjustment by accounting for estimating parameters in GEE
  # est.se is indicator of estimating the asymptotic standard error; when focus in on MSE, don't need se
  
  
  n = nrow(dat)  # original sample size
  dat$PTID <- 1:nrow(dat)
  dat$txAAC <- ifelse(dat$A1==-1, 1, 0)
  
  if (est.weight==TRUE){
    ## by default, weights are estimated with covariates, i.e. ind_prop=0
    # current version of weights estimation:
    # L1 include all baseline cov. and Y0
    # L2 include Y0,Y1
    #L1 <- dat[,c("a.age","a.male","a.ethAfrAmer","a.ethCaucasian","a.ethHispanic","a.ethAsian","Y0")]
    L1 <- dat[,c("a.age","a.male","a.ethAfrAmer","a.ethCaucasian","Y0")]
    L2 <- dat[,c("Y0","Y1")]
    ws1.obj = weight_stage1(dat$A1, L1, 0)
    dat$wt1 <- ws1.obj$wt1
    ws2.obj = weight_stage2(dat$A2, L2, dat$A1, dat$R, 0)
    #### begin debugging ######
    if (sum(ws2.obj$wt2>9998, na.rm=T)>0) {
      ## the warning was already issued, see above.
      return(list(dat2=NULL, gee=NULL, contrasts=matrix(nrow=12, ncol=2, NA), 
                  var=matrix(nrow=10, ncol=10, NA)))   ## may later come back to edit the format of NULL
    }
    #### end debugging ########
    
    dat$wt2 <- ws2.obj$wt2
    dat$wt <- dat$wt1 * dat$wt2
    score_alpha1 = ws1.obj$S_alpha1  ## score_alpha1 and score_alpha2 both have n rows
    score_alpha2 = ws2.obj$S_alpha2    
  }
  if (est.weight==FALSE){
    dat$wt1 <- 2
    dat$wt2 <- ifelse(dat$R==0 & dat$A1==1, 2, 1)
    dat$wt <- dat$wt1 * dat$wt2
  }
  
  
  
  ## convert the data sets from wide to long
  #dat.0 <- dat[,c("PTID","a.age","a.male","a.ethAfrAmer","a.ethCaucasian","a.ethHispanic","a.ethAsian","txAAC","A1","R","A2", "Y0", "Y1", "Y2", "Y3","wt") ]
  dat.0 <- dat[,c("PTID","a.age","a.male","a.ethAfrAmer","a.ethCaucasian","txAAC","A1","R","A2", "Y0", "Y1", "Y2", "Y3","wt") ]
  
  dat.0$Y <- dat.0$Y0
  dat.0$time <- 0.
  #dat.1 <- dat[,c("PTID","a.age","a.male","a.ethAfrAmer","a.ethCaucasian","a.ethHispanic","a.ethAsian","txAAC","A1","R","A2", "Y0", "Y1", "Y2", "Y3","wt") ]
  dat.1 <- dat[,c("PTID","a.age","a.male","a.ethAfrAmer","a.ethCaucasian","txAAC","A1","R","A2", "Y0", "Y1", "Y2", "Y3","wt") ]
  
  dat.1$Y <- dat.1$Y1
  dat.1$time <- 12.
  #dat.2 <- dat[,c("PTID","a.age","a.male","a.ethAfrAmer","a.ethCaucasian","a.ethHispanic","a.ethAsian","txAAC","A1","R","A2", "Y0", "Y1", "Y2", "Y3","wt") ]
  dat.2 <- dat[,c("PTID","a.age","a.male","a.ethAfrAmer","a.ethCaucasian","txAAC","A1","R","A2", "Y0", "Y1", "Y2", "Y3","wt") ]
  
  dat.2$Y <- dat.2$Y2
  dat.2$time <- 24.
  #dat.3 <- dat[,c("PTID","a.age","a.male","a.ethAfrAmer","a.ethCaucasian","a.ethHispanic","a.ethAsian","txAAC","A1","R","A2", "Y0", "Y1", "Y2", "Y3","wt") ]
  dat.3 <- dat[,c("PTID","a.age","a.male","a.ethAfrAmer","a.ethCaucasian","txAAC","A1","R","A2", "Y0", "Y1", "Y2", "Y3","wt") ]
  
  dat.3$Y <- dat.3$Y3
  dat.3$time <- 36.
  dat1 <- rbind(dat.0, dat.1, dat.2, dat.3)
  
  
  ################ UPDATED 03/25/2015 ###################
  dat1$wave <- dat1$time/12 + 1
  #######################################################
  
  
  ## create replications
  ## EMT responders get replicated
  dat2.EMTr.1 <- dat2.EMTr.n1 <- subset(dat1, dat1$R==1 & dat1$A1==1)  
  ## add all PTID in EMTr.n1 by 10000, just so that each PTID is only associated with 4 obs

  ################ UPDATED 03/25/2015 ###################
  ## use the same PTID for replicates
  dat2.EMTr.n1$wave <- dat2.EMTr.n1$wave + 4
  #######################################################
  
  
  dat2.EMTnr  <- subset(dat1, dat1$R==0 & dat1$A1==1)
  dat2.AAC    <- subset(dat1,             dat1$A1==-1)
  ## assign A2=1 vs A2=-1 for EMT responders and concatenate these 2 replicated data sets
  dat2.EMTr.1$A2 <- 1
  dat2.EMTr.n1$A2<- -1
  dat2.EMTr <- rbind(dat2.EMTr.1, dat2.EMTr.n1)
  
  ## create the final analysis data set
  dat2 <- rbind(dat2.EMTr, dat2.EMTnr, dat2.AAC)
  
  
  
  ################ UPDATED 03/25/2015 ###################
  ## sort by PTID  and wave
  ind <- sort.int(as.numeric(dat2$wave), index.return=T)$ix
  dat2 <- dat2[ind,]
  ind <- sort.int(as.numeric(dat2$PTID), index.return=T)$ix
  dat2 <- dat2[ind,]
  ######################################################
  
  
  #### begin debugging ######
  ## added 02-26-2014 by Lucy
  ## rule out the cases where there is no variation in any of the variables in the fitted model
  var.age = var(dat2$a.age)
  var.male = var(dat2$a.male)
  var.ethAfrAmer = var(dat2$a.ethAfrAmer)
  var.ethCaucasian = var(dat2$a.ethCaucasian)
  if (sum((var.age<0.0001), (var.male<0.0001), (var.ethAfrAmer<0.0001), (var.ethCaucasian<0.0001))>0) {
    ## means that at least one of the baseline covariates does not have variation
    ## this could happen in bootstrap samples
    cat("  WARNING: the data set has no variation in at least one baseline variable.\n")
    return(list(dat2=NULL, gee=NULL, contrasts=matrix(nrow=12, ncol=2, NA), 
                var=matrix(nrow = 10, ncol = 10, NA)))
  }
  #### end debugging ####
  
  
  if (type==2) {
    ## want to use exchangeable working correlation
    ## need to properly specify user-defined correlation structure
    
    zcor.unstr <- genZcor(clusz = c(xtabs(~ PTID, data=dat2)), 
                          waves = dat2$wave, corstrv = 4)
    #     zcor.exch <- genZcor(clusz = c(xtabs(~ PTID, data=dat2)), 
    #                          waves = dat2$wave, corstrv = 2)
    #     zcor.ar1 <- genZcor(clusz = c(xtabs(~ PTID, data=dat2)), 
    #                          waves = dat2$wave, corstrv = 3)
    
    zcor.exch <- matrix(NA, nrow(zcor.unstr), 1)
    index.zcorNonzero <- c(1, 2, 3, 8, 9, 14,23:28)
    zcor.exch <- apply(zcor.unstr[,index.zcorNonzero], 1, sum)
    zcor.exch <-  matrix(zcor.exch, ncol=1)
    
    
    
  }
  
  if (model==1) {
    ## fit correct marginal model - force beh regimes to share trajectory until t=12
    ## define some indicators for the piecewise model
    dat2$time.lte.12 <- ifelse(dat2$time<=12, 1, 0)
    dat2$time.gt.12 <- ifelse(dat2$time > 12, 1, 0)
    dat2$cov1       <- dat2$time.lte.12*dat2$time + dat2$time.gt.12*12
    
    
    
    ## fit models
    
    form1 <- " Y ~ a.age + a.male + a.ethAfrAmer + a.ethCaucasian +  
    cov1 + 
    cov1:A1 + 
    time.gt.12:I(time-12) + 
    time.gt.12:I(time-12):A1 +  
    time.gt.12:I(time-12):A2
    "
    form1 <- as.formula(gsub(pattern="\n", replacement="", x=form1))
    #corstring = c("independence","exchangeable")
    corstring = c("independence", "userdefined")  # if non-indep corr is used, must specify "userdefined"
    
    if (type==1) {
      gee <- geeglm( formula = form1,
                     data = dat2,
                     id = PTID,
                     weights = wt,
                     corstr = "independence")
    }
    
    if (type==2) {
      gee <- geeglm( formula = form1,
                     data = dat2,
                     id = PTID,
                     weights = wt,
                     corstr = "userdefined", 
                     zcor=zcor.exch)
    }
    asymvar <- gee$geese$vbeta

    
    

  }
  
  if (model==2) {
    ## fit the slope model - each DTR is modeled to have a linear trajectory
    ## define some indicators for the slope model
    dat2$ind.1.n1 <- ifelse(dat2$A1==1 & dat2$A2==-1, 1, 0)
    dat2$ind.n1 <- ifelse(dat2$A1==-1, 1, 0)
    
    ## fit model
    
    form2 <- " Y ~ a.age + a.male + a.ethAfrAmer + a.ethCaucasian +
    time +
    time:ind.1.n1 +
    time:ind.n1
    "
    form2 <- as.formula(gsub(pattern="\n", replacement="", x=form2))
    corstring = c("independence","exchangeable")
    if (type==1) {
      gee <- geeglm( formula = form2,
                     data = dat2,
                     id = PTID,
                     weights = wt,
                     corstr = "independence")
    }
    
    if (type==2) {
      gee <- geeglm( formula = form2,
                     data = dat2,
                     id = PTID,
                     weights = wt,
                     corstr = "userdefined", 
                     zcor=zcor.exch)
    }
    asymvar <- gee$geese$vbeta
  }
  
  if (model==3) {
    ## fit the quadratic model -- each DTR is modeled to have a quadratic trajectory
    dat2$ind.1.n1 <- ifelse(dat2$A1==1 & dat2$A2==-1, 1, 0)
    dat2$ind.n1 <- ifelse(dat2$A1==-1, 1, 0)
    dat2$timesq <- dat2$time^2
    
    ## fit model
    
    form3 <- " Y ~ a.age + a.male + a.ethAfrAmer + a.ethCaucasian +
    time + timesq +
    time:ind.1.n1 + timesq:ind.1.n1 +
    time:ind.n1 + timesq:ind.n1
    "
    form3 <- as.formula(gsub(pattern="\n", replacement="", x=form3))
    if (type==1) {
      gee <- geeglm( formula = form3,
                     data = dat2,
                     id = PTID,
                     weights = wt,
                     corstr = "independence")
    }
    
    if (type==2) {
      gee <- geeglm( formula = form3,
                     data = dat2,
                     id = PTID,
                     weights = wt,
                     corstr = "userdefined", 
                     zcor=zcor.exch)
    }
    asymvar <- gee$geese$vbeta
    
  }
  
  
  
  ## calculate relevant estimands -- AUC ones and end-of-study ones
  if (model==1){
    # the contrasts under correct marginal model
    # AUCs and contrasts of AUCs
    # AUC.EMT.1   AUC.EMT.n1  AUC.AAC   EMT.1vsEMT.n1   AACvsEMT.n1  AACvsEMT.1
    ## expressed as linear combinations L of interest 
    # the version with all baseline covariates
    
    
    L <- rbind( c(36.0, rep(0,4), 72+24*12, 72+24*12, 24*12, 24*12, 24*12),
                c(36.0, rep(0,4), 72+24*12, 72+24*12, 24*12, 24*12, -24*12),
                c(36.0, rep(0,4), 72+24*12, -72-24*12, 24*12, -24*12, 0.))
    
    L <- rbind(L, L[1,]-L[2,], L[3,]-L[2,], L[3,]-L[1,])
    L <- L/36
    
    Lnames <- c("AUC.EMT.INT",
                "AUC.EMT.AAC",
                "AUC.AAC.AAC",
                
                "EMT.INT.v.EMT.AAC",
                "AAC.AAC.v.EMT.AAC",
                "AAC.AAC.v.EMT.INT"
    )
    rownames(L)<- Lnames
    
    ## end of study mean of Y
    
    L.Y36 <- rbind( c(1, rep(0, 4), 12, 12, 24, 24, 24),
                    c(1, rep(0, 4), 12, 12, 24, 24, -24),
                    c(1, rep(0, 4), 12, -12, 24, -24, 0))
    L.Y36 <- rbind(L.Y36, L.Y36[1,]-L.Y36[2,], L.Y36[3,]-L.Y36[2,], L.Y36[3,]-L.Y36[1,])
    
    Lnames.Y36 <- c("Y36.EMT.INT",
                    "Y36.EMT.AAC",
                    "Y36.AAC.AAC",
                    
                    "Y36.EMT.INT.v.EMT.AAC",
                    "Y36.AAC.AAC.v.EMT.AAC",
                    "Y36.AAC.AAC.v.EMT.INT"
    )
    rownames(L.Y36) <- Lnames.Y36
    
  }
  
  if (model==2){
    ## the contrasts under slope model
    ## AUCs and contrasts of AUCs/36
    # AUC.EMT.1   AUC.EMT.n1  AUC.AAC   EMT.1vsEMT.n1   AACvsEMT.n1  AACvsEMT.1
    L <- rbind(c(1, rep(0,4), 36/2, 0, 0),
               c(1, rep(0,4), 36/2, 36/2, 0),
               c(1, rep(0,4), 36/2, 0, 36/2)
    )
    L <- rbind(L, L[1,]-L[2,], L[3,]-L[2,], L[3,]-L[1,])
    
    Lnames <- c("AUC.EMT.INT",
                "AUC.EMT.AAC",
                "AUC.AAC.AAC",
                
                "EMT.INT.v.EMT.AAC",
                "AAC.AAC.v.EMT.AAC",
                "AAC.AAC.v.EMT.INT"
    )
    rownames(L)<- Lnames
    
    
    
    ## end of study mean of Y
    
    L.Y36 <- rbind( c(1, rep(0,4), 36, 0, 0),
                    c(1, rep(0,4), 36, 36,0),
                    c(1, rep(0,4), 36, 0, 36))
    L.Y36 <- rbind(L.Y36, L.Y36[1,]-L.Y36[2,], L.Y36[3,]-L.Y36[2,], L.Y36[3,]-L.Y36[1,])
    
    Lnames.Y36 <- c("Y36.EMT.INT",
                    "Y36.EMT.AAC",
                    "Y36.AAC.AAC",
                    
                    "Y36.EMT.INT.v.EMT.AAC",
                    "Y36.AAC.AAC.v.EMT.AAC",
                    "Y36.AAC.AAC.v.EMT.INT"
    )
    rownames(L.Y36) <- Lnames.Y36
  }
  
  if (model==3) {
    ## the contrasts under quadratic model
    ## AUCs and contrasts of AUCs/36
    # AUC.EMT.1   AUC.EMT.n1  AUC.AAC   EMT.1vsEMT.n1   AACvsEMT.n1  AACvsEMT.1
    L <- rbind( c(1, rep(0,4), 36/2, 36^2/3, 0, 0, 0, 0),
                c(1, rep(0,4), 36/2, 36^2/3, 36/2, 36^2/3, 0, 0),
                c(1, rep(0,4), 36/2, 36^2/3, 0, 0, 36/2, 36^2/3))
    L <- rbind(L, L[1,]-L[2,], L[3,]-L[2,], L[3,]-L[1,])
    
    Lnames <- c("AUC.EMT.INT",
                "AUC.EMT.AAC",
                "AUC.AAC.AAC",
                
                "EMT.INT.v.EMT.AAC",
                "AAC.AAC.v.EMT.AAC",
                "AAC.AAC.v.EMT.INT"
    )
    rownames(L)<- Lnames
    
    
    ## end of study mean of Y
    
    L.Y36 <- rbind( c(1, rep(0,4), 36, 36^2, 0, 0, 0, 0),
                    c(1, rep(0,4), 36, 36^2, 36, 36^2, 0, 0),
                    c(1, rep(0,4), 36, 36^2, 0, 0, 36, 36^2))
    
    L.Y36 <- rbind(L.Y36, L.Y36[1,]-L.Y36[2,], L.Y36[3,]-L.Y36[2,], L.Y36[3,]-L.Y36[1,])
    
    Lnames.Y36 <- c("Y36.EMT.INT",
                    "Y36.EMT.AAC",
                    "Y36.AAC.AAC",
                    
                    "Y36.EMT.INT.v.EMT.AAC",
                    "Y36.AAC.AAC.v.EMT.AAC",
                    "Y36.AAC.AAC.v.EMT.INT"
    )
    rownames(L.Y36) <- Lnames.Y36
    
  }
  
  
  
  
  # the following part works for all models
  
  
  est <- gee$coef
  
  ## est.L gives parameters of interest: AUCs and contrasts of AUCs
  est.AUC <- L%*%est
  
  ## est.Y36 gives parameters of interest: Y36s and contrasts of Y36s
  est.Y36 <- L.Y36%*%est
  est.L <- rbind(est.Y36, est.AUC)
  
  
  
  if (est.se==TRUE){
    var.Y36 <- L.Y36%*%asymvar%*%t(L.Y36)
    se.Y36 <- sqrt(diag(var.Y36))
    var.AUC <- L%*%asymvar%*%t(L)
    se.AUC <- sqrt(diag(var.AUC))
    
    se.L <- matrix(c(se.Y36, se.AUC), ncol= 1)
    
    contrasts <- cbind(est.L=est.L, se.L=se.L)
    
    #return(list(dat2 = dat2, gee=gee, contrasts=contrasts, var=asymvar, U = U, score1 = score_alpha1, score2 = score_alpha2))
    return(list(dat2 = dat2, gee=gee, contrasts=contrasts, var=asymvar))
  }
  
  if (est.se==FALSE){
    contrasts <- est.L
    return(list(dat2 = dat2, gee=gee, contrasts=contrasts))
    
  }
  
  
}